<?php

class MnglAppController
{
  function MnglAppController()
  {
    add_filter('the_content', array( $this, 'page_route' ), 100);
    add_action('wp_enqueue_scripts', array($this, 'load_scripts'), 1); 
    register_activation_hook(MNGL_PATH."/mingle.php", array( $this, 'install' ));
    
    // Used to process standalone requests
    add_action('init', array($this,'parse_standalone_request'));
    add_action('init', array($this,'mingle_init'));
    
    // Process User registration
    add_filter('pre_user_first_name', array($this, 'add_user_first_name'));
    add_filter('pre_user_last_name', array($this, 'add_user_last_name'));
  }

  function setup_menus()
  {
    add_action('admin_menu', array( $this, 'menu' ));
  }
  
  /********* INSTALL PLUGIN ***********/
  function install()
  {
    global $wpdb;
    $db_version = 5; // this is the version of the database we're moving to
    $old_db_version = get_option('mngl_db_version');

    $friends_table           = $wpdb->prefix . "mngl_friends";
    $friend_requests_table   = $wpdb->prefix . "mngl_friend_requests";
    $board_posts_table       = $wpdb->prefix . "mngl_board_posts";
    $board_comments_table    = $wpdb->prefix . "mngl_board_comments";

    $charset_collate = '';
    if( $wpdb->has_cap( 'collation' ) )
    {
      if( !empty($wpdb->charset) )
        $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
      if( !empty($wpdb->collate) )
        $charset_collate .= " COLLATE $wpdb->collate";
    }

    if($db_version != $old_db_version)
    {
      $this->before_migration($old_db_version);
      
      require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

      /* Create/Upgrade Friends Table */
      $sql = "CREATE TABLE {$friends_table} (
                id int(11) NOT NULL auto_increment,
                user_id int(11) NOT NULL,
                friend_id int(11) NOT NULL,
                status varchar(255) DEFAULT NULL,
                created_at datetime NOT NULL,
                PRIMARY KEY  (id),
                KEY user_id (user_id),
                KEY friend_id (friend_id),
                KEY status (status)
              ) {$charset_collate};";
    
      dbDelta($sql);
      
      /* Create/Upgrade Friend Requests Table */
      $sql = "CREATE TABLE {$friend_requests_table} (
                id int(11) NOT NULL auto_increment,
                user_id int(11) NOT NULL,
                friend_id int(11) NOT NULL,
                friend_record_a_id int(11) NOT NULL,
                friend_record_b_id int(11) NOT NULL,
                created_at datetime NOT NULL,
                PRIMARY KEY  (id),
                KEY user_id (user_id),
                KEY friend_id (friend_id),
                KEY friend_record_a_id (friend_record_a_id),
                KEY friend_record_b_id (friend_record_b_id)
              ) {$charset_collate};";
    
      dbDelta($sql);

      /* Create/Upgrade Board Posts Table */
      $sql = "CREATE TABLE {$board_posts_table} (
                id int(11) NOT NULL auto_increment,
                owner_id int(11) NOT NULL,
                author_id int(11) NOT NULL,
                message text DEFAULT NULL,
                type varchar(255) DEFAULT 'post',
                source varchar(255) DEFAULT NULL,
                visibility varchar(255) DEFAULT 'public',
                created_at datetime NOT NULL,
                PRIMARY KEY  (id),
                KEY owner_id (owner_id),
                KEY author_id (author_id),
                KEY type (type),
                KEY source (source),
                KEY visibility (visibility)
              ) {$charset_collate};";
    
      dbDelta($sql);
      
      /* Create/Upgrade Board Comments Table */
      $sql = "CREATE TABLE {$board_comments_table} (
                id int(11) NOT NULL auto_increment,
                author_id int(11) NOT NULL,
                message text DEFAULT NULL,
                board_post_id int(11) NOT NULL,
                created_at datetime NOT NULL,
                PRIMARY KEY  (id),
                KEY author_id (author_id),
                KEY board_post_id (board_post_id)
              ) {$charset_collate};";
    
      dbDelta($sql);

      $this->after_migration($old_db_version);
    }

    /***** SAVE DB VERSION *****/
    delete_option('mngl_db_version');
    add_option('mngl_db_version',$db_version);
  }
  
  function before_migration($curr_db_version)
  {
    // Nothing here yet
  }
  
  function after_migration($curr_db_version)
  {
    if(isset($curr_db_version) and !empty($curr_db_version))
    {
      if($curr_db_version < 4)
      {
        global $mngl_options;
        
        if($mngl_options->display_name_type == 'fullname')
        {
          global $wpdb;

          $query = "SELECT ID FROM {$wpdb->prefix}users";
          $user_ids = $wpdb->get_col($query,0);
          
          foreach ($user_ids as $user_id)
          {
            $profile = MnglUser::get_stored_profile_by_id($user_id);
            if( isset($profile->first_name) and !empty($profile->first_name) and
                isset($profile->last_name) and !empty($profile->last_name) )
            {
              $profile->full_name = "{$profile->first_name} {$profile->last_name}";
              $profile->store(true);
            }
            else if( isset($profile->first_name) and !empty($profile->first_name) )
            {
              $profile->full_name = $profile->first_name;
              $profile->store(true);
            }
            else if( isset($profile->last_name) and !empty($profile->last_name) )
            {
              $profile->full_name = $profile->last_name;
              $profile->first_name = $profile->last_name;
              $profile->store(true);
            }
          }
        }
      }
    }
  }
  
  function menu()
  {
    global $options_controller, $mngl_update;
  
    add_menu_page(__('Mingle', 'mingle'), __('Mingle', 'mingle'), 8, 'mingle-options', array($options_controller,'route'), MNGL_URL . "/images/mingle_16.png");
    //add_submenu_page( 'mingle-options', __('Mingle Plus'), __('Mingle Plus'), 8, 'mingle-pro-settings', array( $mngl_update, 'pro_cred_form') );
  }
  
  // Routes for wordpress pages -- we're just replacing content here folks.
  function page_route($content)
  {
    global $post, $mngl_options, $profiles_controller, $boards_controller, $friends_controller, $mngl_board_post;
    
    switch( $post->ID )
    {  
      case $mngl_options->activity_page_id:
        // Start output buffering -- we want to return the output as a string
        ob_start();
        $profiles_controller->activity();
        // Pull all the output into this variable
        $content = ob_get_contents();
        // End and erase the output buffer (so we control where it's output)
        ob_end_clean();
        break;
      case $mngl_options->profile_page_id:
        ob_start();
        if($this->get_param('mbpost'))
          $boards_controller->display_board_post($mngl_board_post->get_one($this->get_param('mbpost'),true));
        else
          $profiles_controller->profile();
        $content = ob_get_contents();
        ob_end_clean();
        break;
      case $mngl_options->directory_page_id:
        ob_start();
        $profiles_controller->directory($this->get_param('mdp'));
        $content = ob_get_contents();
        ob_end_clean();
        break;
      case $mngl_options->profile_edit_page_id:
        ob_start();
        $profiles_controller->edit();
        $content = ob_get_contents();
        ob_end_clean();
        break;
      case $mngl_options->friends_page_id:
        ob_start();
        $friends_controller->list_friends($this->get_param('mdp'), $this->get_param('u'));
        $content = ob_get_contents();
        ob_end_clean();
        break;
      case $mngl_options->friend_requests_page_id:
        ob_start();
        $friends_controller->list_friend_requests();
        $content = ob_get_contents();
        ob_end_clean();
        break;
      case $mngl_options->login_page_id:
        ob_start();
        $this->display_login_form($this->get_param('redirect_to'));
        $content = ob_get_contents();
        ob_end_clean();
        break;
      case $mngl_options->signup_page_id:
        ob_start();
        $this->display_signup_form();
        $content = ob_get_contents();
        ob_end_clean();
        break;
    }
    
    return $content;
  }  

  function load_scripts()
  {
    global $mngl_blogurl, $post, $mngl_options;
    
    /* TODO: For when we get private messaging going:
    wp_enqueue_style( 'mingle',  MNGL_CSS_URL . '/mingle.css' );
    wp_enqueue_style( 'autocomplete',  MNGL_CSS_URL . '/jquery.autocompletefb.css' );
    wp_enqueue_script( 'bgiframe', MNGL_JS_URL . '/jquery.bgiframe.min.js', array('jquery') );
    wp_enqueue_script( 'dimensions', MNGL_JS_URL . '/jquery.dimensions.js', array('bgiframe') );
    wp_enqueue_script( 'autocomplete', MNGL_JS_URL . '/jquery.autocomplete.js', array('dimensions') );
    wp_enqueue_script( 'autocompletefb', MNGL_JS_URL . '/jquery.autocompletefb.js', array('autocomplete') );
    wp_enqueue_script( 'mingle', MNGL_SCRIPT_URL . '&controller=js&action=mingle', array('autocompletefb') );
    */
    wp_enqueue_style( 'mingle',  MNGL_CSS_URL . '/mingle.css' );
    wp_enqueue_script( 'jquery-elastic', MNGL_JS_URL . '/jquery.elastic.js', array('jquery') );
    wp_enqueue_script( 'mingle', $mngl_blogurl . '/index.php?mingle_js=mingle', array('jquery','jquery-elastic') );
  }
  
  function mingle_js()
  {
    header('Content-type: application/javascript');
    require_once( MNGL_JS_PATH . '/mingle.js.php' );
  }

  // The tight way to process standalone requests dogg...
  function parse_standalone_request()
  {
    $plugin     = $this->get_param('plugin');
    $action     = $this->get_param('action');
    $controller = $this->get_param('controller');
    $mingle_js  = $this->get_param('mingle_js');
    
    if( !empty($plugin) and $plugin == 'mingle' and 
        !empty($controller) and !empty($action) )
    {
      $this->standalone_route($controller, $action);
      exit;
    }
    else if( !empty($mingle_js) ) // load js seperately because of enqueue script url encoding weirdness
    {
      $this->standalone_route('js', $mingle_js);
      exit;
    }
  }
  
  // Routes for standalone / ajax requests
  function standalone_route($controller, $action)
  {
    global $friends_controller, $boards_controller, $profiles_controller, $options_controller;
    
    if($controller=='friends')
    {
      if($action=='friend_request')
        $friends_controller->friend_request($this->get_param('user_id'), $this->get_param('friend_id'));
      if($action=='delete_friend')
        $friends_controller->delete_friend($this->get_param('user_id'), $this->get_param('friend_id'));
      else if($action=='accept_friend')
        $friends_controller->accept_friend($this->get_param('request_id'));
      else if($action=='ignore_friend')
        $friends_controller->ignore_friend($this->get_param('request_id'));
      else if($action=='search')
        $friends_controller->list_friends($this->get_param('mdp'),$this->get_param('u'),true,$this->get_param('q'));
    }
    else if($controller=='boards')
    {
      if($action=='post')
        $boards_controller->post($this->get_param('owner_id'), $this->get_param('author_id'), $this->get_param('message'));
      else if($action=='comment')
        $boards_controller->comment($this->get_param('board_post_id'), $this->get_param('author_id'), $this->get_param('message'));
      else if($action=='delete_post')
        $boards_controller->delete_post($this->get_param('board_post_id'));
      else if($action=='delete_comment')
        $boards_controller->delete_comment($this->get_param('board_comment_id'));
      else if($action=='older_posts')
        $boards_controller->show_older_posts($this->get_param('u'),$this->get_param('mdp'),$this->get_param('loc'));
    }
    else if($controller=='activity')
    {
      if($action=='post')
        $boards_controller->post($this->get_param('owner_id'), $this->get_param('author_id'), $this->get_param('message'),true);
      else if($action=='comment')
        $boards_controller->comment($this->get_param('board_post_id'), $this->get_param('author_id'), $this->get_param('message'),true);
      else if($action=='delete_post')
        $boards_controller->delete_post($this->get_param('board_post_id'),true);
      else if($action=='delete_comment')
        $boards_controller->delete_comment($this->get_param('board_comment_id'),true);
    }
    else if($controller=='profile')
    {  
      if($action=='delete_avatar')
        $profiles_controller->delete_avatar($this->get_param('user_id'));
      else if($action=='search')
        $profiles_controller->directory($this->get_param('mdp'),true,$this->get_param('q'));
    }
    else if($controller=='options')
    {
      if($action=='add_default_user')
        $options_controller->display_default_friend_drop_down();
    }
    else if($controller=='js')
    {
      if($action=='mingle')
        $this->mingle_js();
    }
  }
  
  function load_language()
  {
    $path_from_plugins_folder = str_replace( ABSPATH, '', MNGL_PATH ) . '/i18n/';
    
    load_plugin_textdomain( 'mingle', $path_from_plugins_folder );
  }
  
  function mingle_init()
  {
  	add_filter('get_avatar', array($this,'override_avatar'), 10, 4);
    add_filter('get_comment', array($this,'override_comment_author_details'));
  }

  function override_comment_author_details($comment)
  {
    $user_id = (int)MnglUtils::get_user_id_by_email($comment->comment_author_email);
    
    if($user_id and !empty($user_id))
    {
      $user = MnglUser::get_stored_profile_by_id($user_id);
      $comment->comment_author_url = $user->get_profile_url();
    }
    
    return $comment;
  }
  
  function override_avatar($avatar, $id_or_email, $size, $default)
  {
    $user_id = false;

  	if( is_object($id_or_email) )
  	{
  		if( $id_or_email->ID )
  			$user_id = $id_or_email->ID;
  		else if($id_or_email->user_id)
  			$user_id = $id_or_email->user_id;
  		else if($id_or_email->comment_author_email)
  			$user_id = (int)MnglUtils::get_user_id_by_email($id_or_email->comment_author_email);
  	}
  	else if( is_numeric($id_or_email) )
  		$user_id = (int)$id_or_email;
  	else if( is_string($id_or_email) )
  		$user_id = (int)MnglUtils::get_user_id_by_email($id_or_email);

  	if(!$user_id or empty($user_id))
  		return $avatar;
    
    preg_match('/class=["\'](.*?)["\']/', $avatar, $class_match);

    $additional_classes = ((isset($class_match[1]))?$class_match[1]:'');

    $new_avatar = MnglAppHelper::get_avatar_link_by_id($user_id, $size, $size, $additional_classes, true);
    
    return $new_avatar;
  }

  // Utility function to grab the parameter whether it's a get or post
  function get_param($param)
  {
    return (isset($_POST[$param])?$_POST[$param]:$_GET[$param]);
  }
  
  function get_param_delimiter_char($link)
  { 
    return ((preg_match("#\?#",$link))?'&':'?');
  }
  
  function display_login_form($redirect_to)
  {
    global $mngl_options, $mngl_blogurl;
    
    $redirect_to = ((!empty($redirect_to))?$redirect_to:get_permalink($mngl_options->activity_page_id));

    if(!empty($mngl_options->signup_page_id) and $mngl_options->signup_page_id > 0)
      $signup_url = get_permalink($mngl_options->signup_page_id);
    else
      $signup_url = $mngl_blogurl . '/wp-login.php?action=register';
    
    if(is_user_logged_in()) 
      require( MNGL_VIEWS_PATH . '/shared/already_logged_in.php' );
    else
      require( MNGL_VIEWS_PATH . '/shared/login_form.php' );
  }
  
  function display_signup_form()
  {
    global $mngl_options, $mngl_blogurl;
    
    if(is_user_logged_in()) 
      require( MNGL_VIEWS_PATH . '/shared/already_logged_in.php' );
    else
      require( MNGL_VIEWS_PATH . '/shared/signup_form.php' );
  }
  
  function add_user_first_name($first_name)
  {
    if(empty($first_name) and !empty($_POST['mngl_user_first-name']))
      $first_name = $_POST['mngl_user_first-name'];
      
    return $first_name;
  }
  
  function add_user_last_name($last_name)
  {
    if(empty($last_name) and !empty($_POST['mngl_user_last-name']))
      $last_name = $_POST['mngl_user_last-name'];
      
    return $last_name;
  }
}
?>
